@@ -5,7 +5,7 @@
     sosw - Serverless Orchestrator of Serverless Workers
 
     The MIT License (MIT)
-    Copyright (C) 2022  sosw core contributors <info@sosw.app>
+    Copyright (C) 2024  sosw core contributors <info@sosw.app>
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to deal
@@ -25,28 +25,76 @@
     OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
     SOFTWARE.
 
-Config manager component. Has methods for getting configuration for lambdas.
-Use `Config` class to get the correct methods we use for each action.
-Using SSMConfig or DynamoConfig directly is discouraged.
-Especially SSMConfig, because SSM throttles and has limits we reached in the past.
-Using these methods requires the Role to have permissions to access SSM and/or Dynamo for requested resources.
+Config manager component. Has methods for getting configuration for Lambdas.
+
+The default is ``DynamoConfig``. It is automatically called from the ``app.Processor.__init__`` and if your Lambda
+has permissions to access  the ``config`` table, the Processor will look for the record: ``YOUR_FUNCTION_NAME_config``,
+and recursively  update the ``DEFAULT_CONFIG`` with it.
+
+You can also import and use the following functions.
+They can be directly imported from this module and will be automatically switched to DDB / SSM / Secrets manager.
+
+- get_config_
+- get_credentials_by_prefix_
+- get_secrets_credentials_
+- update_config_
+
+..  warning::
+
+    Using these methods requires the Role to have relevant permissions to access DynamoDB / SSM Parameter Store /
+    AWS Secrets.
+
+Usage example `(pseudo code)`:
+
+..  code-block:: python
+
+    from SOME_DB_DRIVER import connect
+    from sosw.components.config import get_secrets_credentials, get_config, update_config
+    from sosw.app import Processor as SoswProcessor
+
+    class Processor(SoswProcessor):
+
+        def work_with_db(self):
+
+            db_settings = get_credentials_by_prefix('db_')
+            db_password = get_secrets_credentials(type='name', value='db_password')['db_password']
+
+            connection = connect(**db_settings, password=db_password)
+
+            last_processed_row = get_config('last_row')
+
+            db_result = connection.query(f"SOME QUERY LIMIT last_processed_row, 10;")
+            self.do_something(db_result)
+
+            update_config('last_row', last_processed_row + 10)
+
 """
 
-__all__ = ['ConfigSource', 'get_config', 'update_config', 'get_credentials_by_prefix']
+__all__ = ['ConfigSource', 'get_config', 'update_config', 'get_credentials_by_prefix', 'get_secrets_credentials']
 __author__ = "Sophie Fogel, Nikolay Grishchenko"
-__version__ = "1.7.1"
+__version__ = "1.7.3"
+
+try:
+    from aws_lambda_powertools import Logger
+
+    logger = Logger(child=True)
+
+except ImportError:
+    import logging
+
+    logger = logging.getLogger()
+    logger.setLevel(logging.INFO)
 
 import boto3
 import json
-import logging
 import os
 
 from sosw.components.helpers import chunks
 from sosw.components.dynamo_db import DynamoDbClient
+from sosw.test.helpers_test_dynamo_db import get_autotest_ddb_name_with_custom_suffix
 
 
 class SecretsManager:
-
     secretsmanager_client = None
 
 
@@ -66,15 +114,16 @@ class SecretsManager:
         return self.secretsmanager_client
 
 
-    def call_boto_secrets_with_pagination(self, f, **kwargs):
+    def call_boto_secrets_with_pagination(self, f: str, **kwargs) -> list:
         """
         Invoke SecretsManager functions with the ability to paginate results.
 
-        :param str f:           SecretsManager function to invoke.
-        :param object kwargs:   Keyword arguments for the function to invoke.
-        :rtype list
-        :return:                If the function can be the paginate the response will return as paginate iterator.
-                                Else it will return as a list
+        ..  _get_secrets_credentials:
+
+        :param f:   SecretsManager function to invoke.
+
+        :return:    If the function can be the paginate the response will return as paginate iterator.
+                    Else it will return as a list
         """
 
         secretsmanager_client = self._get_secretsmanager_client()
@@ -83,13 +132,13 @@ class SecretsManager:
         can_paginate = getattr(secretsmanager_client, 'can_paginate')(f)
 
         if can_paginate:
-            logging.debug("SecretsManager.%s can natively paginate", f)
+            logger.debug("SecretsManager.%s can natively paginate", f)
             paginator = secretsmanager_client.get_paginator(f)
             response = paginator.paginate(**kwargs)
             return response
 
         else:
-            logging.debug("SecretsManager.%s can not natively paginate", f)
+            logger.debug("SecretsManager.%s can not natively paginate", f)
             response_list = []
             response = func(**kwargs)
             response_list.append(response)
@@ -99,12 +148,20 @@ class SecretsManager:
             return response_list
 
 
-    def get_secrets_credentials(self, **kwargs):
+    def get_secrets_credentials(self, **kwargs) -> dict:
         """
+
         Retrieve the credentials with given name or tag from AWS SecretsManager and return as a dictionary.
-        :param  kwargs:  {type: tag/name, value: value_name}
-        :rtype:             dict
-        :return:            Some credentials
+
+        Must provide ``type`` and ``value`` to search. Type is either ``'tag'`` or ``'name'``. ``value`` is a string.
+
+        Usage example:
+
+        ..  code-block:: python
+
+            my_secret =  get_secrets_credentials(type='name', value='my_secret_name')
+
+            my_secrets_by_tag = get_secrets_credentials(type='tag', value='project_a_credentials')
         """
 
         filters, secrets_dict = [], {}
@@ -130,11 +187,12 @@ class SecretsManager:
                 secret_value = secretsmanager_client.get_secret_value(SecretId=secret['ARN'])
                 secrets_dict[secret['Name']] = secret_value['SecretString']
         else:
-            logging.warning('No credentials found in SecretsManager for %s with %s', filter_type, value)
+            logger.warning('No credentials found in SecretsManager for %s with %s', filter_type, value)
             return secrets_dict
 
         return secrets_dict
 
+
 class SSMConfig:
     """
     Methods to access some configurations and/or credentials stored in AWS SSM ParameterStore.
@@ -174,13 +232,13 @@ class SSMConfig:
 
         try:
             response = ssm_client.get_parameters(
-                    Names=[name],
-                    WithDecryption=True
+                Names=[name],
+                WithDecryption=True
             )
         except Exception:
             response = ssm_client.get_parameters(
-                    Names=[name],
-                    WithDecryption=False
+                Names=[name],
+                WithDecryption=False
             )
 
         try:
@@ -209,22 +267,20 @@ class SSMConfig:
 
         ssm_client = self._get_ssm_client()
         ssm_client.put_parameter(
-                Name=name,
-                Description=description,
-                Value=val,
-                Type=param_type,
-                Overwrite=True
+            Name=name,
+            Description=description,
+            Value=val,
+            Type=param_type,
+            Overwrite=True
         )
 
 
-    def call_boto_with_pagination(self, f, **kwargs):
+    def call_boto_with_pagination(self, f, **kwargs) -> list:
         """
         Invoke SSM functions with the ability to paginate results.
 
         :param str f:           SSM function to invoke.
         :param object kwargs:   Keyword arguments for the function to invoke.
-        :rtype list
-        :return:                List of paginated responses.
         """
 
         ssm_client = self._get_ssm_client()
@@ -233,13 +289,13 @@ class SSMConfig:
         can_paginate = getattr(ssm_client, 'can_paginate')(f)
 
         if can_paginate:
-            logging.debug(f"'SSM.{f}()' can natively paginate")
+            logger.debug("'SSM.%s()' can natively paginate", f)
             paginator = ssm_client.get_paginator(f)
             response = paginator.paginate(**kwargs)
             return list(response)
 
         else:
-            logging.debug(f"'SSM.{f}()' can not natively paginate")
+            logger.debug("'SSM.%s()' can not natively paginate", f)
             response_list = []
             response = func(**kwargs)
             response_list.append(response)
@@ -272,13 +328,13 @@ class SSMConfig:
                                                                           'Values': [prefix]
                                                                       }])
 
-        logging.debug(f"SSM.describe_parameters(prefix={prefix}) received response: {describe_params_response}")
+        logger.debug("SSM.describe_parameters(prefix=%s) received response: %s", prefix, describe_params_response)
         params = [param for obj in describe_params_response for param in obj['Parameters']]
 
         names = [param['Name'] for param in params]
         if not names:
-            logging.warning(
-                    f"No credentials found in SSM ParameterStore with prefix {prefix} for Environment: {env_tag}")
+            logger.warning(
+                "No credentials found in SSM ParameterStore with prefix %s for Environment: %s", prefix, env_tag)
             return dict()
 
         # This is supposed to work fine if you ask multiple keys even if some are not encrypted.
@@ -289,7 +345,7 @@ class SSMConfig:
         for chunk_of_names in chunks(names, 10):
             get_params_response = self.call_boto_with_pagination('get_parameters', Names=chunk_of_names,
                                                                  WithDecryption=decryption_required)
-            logging.debug(f"SSM.get_parameters(names={chunk_of_names}) received response: {get_params_response}")
+            logger.debug(f"SSM.get_parameters(names=%s) received response: %s", chunk_of_names, get_params_response)
 
             # Update keys and values from this page of response to result. Removes the prefix away for keys.
             params = [param for obj in get_params_response for param in obj['Parameters']]
@@ -302,10 +358,23 @@ class SSMConfig:
 
 class DynamoConfig:
     """
-    Methods to get/update config from/to DynamoDB.
+    This is a manager to operate with custom configurations for Lambdas stored in the ``config`` DynamoDB table.
+    It tries to find the table ``config`` in the same region where the Lambda runs with the following structure:
+
+    ..  code-block:: python
+
+        'env':          'S',
+        'config_name':  'S',
+        'config_value': 'S',
+
+    If the table exists and the Lambda has permissions to access it, the class will look for the record:
+    ``YOUR_FUNCTION_config``, and recursively update the ``DEFAULT_CONFIG`` with it.
+
+    .. _get_config:
     """
 
-    dynamo_client = None
+    dynamo_client: DynamoDbClient = None
+    no_ddb_access: bool = None
 
 
     def __init__(self, **kwargs):
@@ -332,31 +401,44 @@ class DynamoConfig:
 
     def get_config(self, name, env="production"):
         """
-        Retrieve the Config from DynamoDB 'config' table and return as a JSON parsed dictionary.
+        Retrieve the Config from DynamoDB ``config`` table and return as a JSON parsed dictionary.
         If not in JSON format, returns a string.
+
+        ..  note::
+
+            In case the environment is ``test`` the table name will be automatically changed to ``autotest_config``.
+            This might be relevant only for complex integration tests.
+
         :param str name:    Name of config to extract
-        :param str env:     Environment the variable belongs to: 'production' or 'dev'
+        :param str env:     Environment name, usually: 'production' or 'dev'
         :rtype:             dict|string
         :return:            Configuration
+
+        ..  _update_config:
         """
 
-        dynamo_client = self._get_dynamo_client()
-        if os.environ.get('STAGE') == 'test' or os.environ.get('autotest') == 'True':
-            dynamo_client.config['table_name'] = 'autotest_config'
+        if dynamo_client := self._get_dynamo_client():
 
-        items = dynamo_client.get_by_query(keys={'env': env, 'config_name': name})
-        item = items[0] if items else None
-        config_value = item.get('config_value') if item else None
-
-        try:
-            return json.loads(config_value)
-        except Exception:
-            return config_value if config_value is not None else {}
+            items = dynamo_client.get_by_query(keys={'env': env, 'config_name': name})
+            item = items[0] if items else None
+            config_value = item.get('config_value') if item else None
+            logger.debug("Got config value from DDB: %s", config_value)
+            try:
+                return json.loads(config_value)
+            except (json.JSONDecodeError, TypeError) as err:
+                return config_value if config_value is not None else {}
+        else:
+            logger.info("Tried to get DynamoConfig, but failed.")
+        return {}
 
 
     def update_config(self, name, val, **kwargs):
         """
-        Update a field in DynamoDB 'config' table with a new value.
+        Update a field in the DynamoDB ``config`` table with a new value. May be used to store not sensitive tokens.
+
+        ..  warning: For sensitive credentials use SecretsManager!
+
+        ..  _get_credentials_by_prefix:
 
         :param  str     name:   Field name to address.
         :param  object  val:    Field value to update.
@@ -371,37 +453,61 @@ class DynamoConfig:
         dynamo_client.update(keys={'env': env, 'config_name': name}, attributes_to_update={'config_value': val})
 
 
-    def get_credentials_by_prefix(self, prefix, env="production"):
+    def get_credentials_by_prefix(self, prefix: str, env: str = 'production') -> dict:
+        """
+        Fetches multiple records from the ``config`` table. Filters rows that start with the prefix and returns them
+        as a dict with this prefix trimmed.
+
+        ..  warning: For sensitive credentials use SecretsManager!
+
+        Example in ``config`` DDB:
+
+        ..  code-block:: python
+
+            {'env': 'production', 'config_name': 'project_a_db_username', 'config_value': 'john'}
+            {'env': 'production', 'config_name': 'project_a_db_port', 'config_value': '27019'}
+            {'env': 'production', 'config_name': 'another_db_username', 'config_value': 'silver'}
+
+
+            credentials = get_credentials_by_prefix('project_a_db')
+
+            # {'username': 'john', 'port': '27019'}
+        """
+
 
         prefix = prefix if prefix.endswith('_') else prefix + '_'
 
         if self.test or prefix.startswith('autotest_'):
             env = "dev"
 
-        dynamo_client = self._get_dynamo_client()
-        items = dynamo_client.get_by_query(keys={'env': env, 'config_name': prefix},
-                                           comparisons={'config_name': 'begins_with'})
-
         res = {}
-        for row in items:
-            try:
-                row['config_value'] = json.loads(row['config_value'])
-            except Exception:
-                pass
-            config_name = row['config_name'].replace(prefix, '')
-            res[config_name] = row['config_value']
+        if dynamo_client := self._get_dynamo_client():
+            items = dynamo_client.get_by_query(keys={'env': env, 'config_name': prefix},
+                                               comparisons={'config_name': 'begins_with'})
+            for row in items:
+                try:
+                    row['config_value'] = json.loads(row['config_value'])
+                except (json.JSONDecodeError, TypeError):
+                    pass
+                config_name = row['config_name'].replace(prefix, '')
+                res[config_name] = row['config_value']
 
         return res
 
 
     def _get_dynamo_client(self):
-        if self.dynamo_client is None:
+        if self.dynamo_client is None and not self.no_ddb_access:
             dynamo_config = self.config.get('dynamo_client_config')
 
+            # FIXME Consider deprecating this. Used only for safety, but hardcoding is bad.
             if self.test:
-                dynamo_config['table_name'] = 'autotest_config_component'
+                dynamo_config['table_name'] = get_autotest_ddb_name_with_custom_suffix('config')
 
-            self.dynamo_client = DynamoDbClient(dynamo_config)
+            try:
+                self.dynamo_client = DynamoDbClient(dynamo_config)
+            except Exception as err:
+                logger.warning("Failed to initialize DynamoDB client for ConfigSource: %s", err)
+                self.no_ddb_access = True
 
         return self.dynamo_client
 
@@ -421,7 +527,7 @@ class ConfigSource:
     SUPPORTED_SOURCES = ('Dynamo', 'SSM')
 
 
-    def __init__(self, test=False, sources=None, config=None):
+    def __init__(self, test=False, sources=None, config=None, **kwargs):
 
         self.test = test or True if os.environ.get('STAGE') == 'test' else False
 
@@ -448,38 +554,39 @@ class ConfigSource:
             cfg = self.config.get(f"{source.lower()}_config", {})
 
             # Config Client class
-            cls = globals()[f"{source}Config"](config=cfg, test=self.test)
+            cls = globals()[f"{source}Config"](config=cfg, test=self.test, **kwargs)
 
             # Set instance of Config Client as attribute of current ConfigSource object.
             setattr(self, f"{source.lower()}_config", cls)
 
             if not self.default_source:
                 self.default_source = getattr(self, f"{source.lower()}_config")
-                logging.info(f"Initialized default_source = {source.lower()}_config")
+                logger.info("Initialized default_source = %s_config", source.lower())
 
         self.secrets_manager_class = SecretsManager()
 
-    def get_config(self, name):
-        return self.default_source.get_config(name)
+
+    def get_config(self, name, **kwargs):
+        return self.default_source.get_config(name, **kwargs)
 
 
     def update_config(self, name, val, **kwargs):
         return self.default_source.update_config(name, val, **kwargs)
 
 
-    def get_credentials_by_prefix(self, prefix):
-        return self.default_source.get_credentials_by_prefix(prefix)
+    def get_credentials_by_prefix(self, prefix, **kwargs):
+        return self.default_source.get_credentials_by_prefix(prefix, **kwargs)
+
 
     def get_secrets_credentials(self, **kwargs):
         return self.secrets_manager_class.get_secrets_credentials(**kwargs)
 
 
-
 test = True if os.environ.get('STAGE') == 'test' else False
 
-
 __config_source = ConfigSource(test=test)
 
 get_config = __config_source.get_config
 update_config = __config_source.update_config
 get_credentials_by_prefix = __config_source.get_credentials_by_prefix
+get_secrets_credentials = __config_source.get_secrets_credentials
