@@ -5,7 +5,7 @@
     sosw - Serverless Orchestrator of Serverless Workers
 
     The MIT License (MIT)
-    Copyright (C) 2022  sosw core contributors <info@sosw.app>
+    Copyright (C) 2024  sosw core contributors <info@sosw.app>
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to deal
@@ -30,9 +30,19 @@ __all__ = ['DynamoDbClient', 'clean_dynamo_table']
 __author__ = "Nikolay Grishchenko, Sophie Fogel, Gil Halperin, Mark Bulgakov"
 __version__ = "1.7"
 
+try:
+    from aws_lambda_powertools import Logger
+
+    logger = Logger(child=True)
+
+except ImportError:
+    import logging
+
+    logger = logging.getLogger()
+    logger.setLevel(logging.INFO)
+
 import boto3
 import datetime
-import logging
 import json
 import os
 import time
@@ -46,10 +56,6 @@ from .benchmark import benchmark
 from .helpers import chunks, to_bool
 
 
-logger = logging.getLogger()
-logger.setLevel(logging.INFO)
-
-
 class DynamoDbClient:
     """
     Has default methods for different types of DynamoDB tables.
@@ -60,7 +66,7 @@ class DynamoDbClient:
     Config should have a mapping for the field types and required fields.
     Config example:
 
-    .. code-block:: python
+    ..  code-block:: python
 
         {
             'row_mapper':     {
@@ -116,9 +122,9 @@ class DynamoDbClient:
         # Use the config value if not provided
         if table_name is None:
             table_name = self.config['table_name']
-            logging.debug("Got `table_name` from config: %s", table_name)
+            logger.debug("Got ``table_name`` from config: %s", table_name)
 
-        logging.debug("DynamoDB table name identified as %s", table_name)
+        logger.debug("DynamoDB table name identified as %s", table_name)
 
         # Fetch the actual configuration of the dynamodb table directly for
         table_description = self._describe_table(table_name)
@@ -198,7 +204,7 @@ class DynamoDbClient:
 
         ..  note::
 
-            In case the table has ON DEMAND (PAY_PER_REQUEST) BillingMode the provisioned_throughput is missing.
+            In case the table has ON DEMAND (PAY_PER_REQUEST) BillingMode the provisioned_throughput is set to 0.
         """
 
         indexes = {}
@@ -254,8 +260,8 @@ class DynamoDbClient:
         We currently support only String or Numeric values. Latest ones are converted to int or float.
         Takes settings from row_mapper.
 
-        e.g.:               {'key1': {'N': '3'}, 'key2': {'S': 'value2'}}
-        will convert to:    {'key1': 3, 'key2': 'value2'}
+        e.g.:               ``{'key1': {'N': '3'}, 'key2': {'S': 'value2'}}``
+        will convert to:    ``{'key1': 3, 'key2': 'value2'}``
 
         :param dict dynamo_row:       DynamoDB row item
         :param bool strict:           DEPRECATED.
@@ -266,8 +272,8 @@ class DynamoDbClient:
         """
 
         if strict is not None:
-            logging.warning(f"dynamo_to_dict `strict` variable is deprecated in sosw 0.7.13+. "
-                            f"Please replace it's usage with `fetch_all_fields` (and reverse the boolean value)")
+            logger.warning("dynamo_to_dict ``strict`` variable is deprecated in sosw 0.7.13+. "
+                           "Please replace it's usage with ``fetch_all_fields`` (and reverse the boolean value)")
         fetch_all_fields = fetch_all_fields if fetch_all_fields is not None else False if strict is None else not strict
         result = {}
 
@@ -283,7 +289,7 @@ class DynamoDbClient:
                         raise ValueError(f"'{key}' is expected to be of type '{key_type}' in row_mapper, "
                                          f"but real value is of type '{real_type}'")
 
-                    # type_deserializer.deserialize() parses 'N' to `Decimal` type but it cant be parsed to a datetime
+                    # type_deserializer.deserialize() parses 'N' to ``Decimal`` type but it cant be parsed to a datetime
                     # so we cast it to either an integer or a float.
                     if key_type == 'N':
                         result[key] = float(val) if '.' in val else int(val)
@@ -296,7 +302,7 @@ class DynamoDbClient:
                             try:
                                 result[key] = json.loads(val)
                             except ValueError:
-                                logger.warning(f"A JSON-looking string failed to parse: {val}")
+                                logger.warning("A JSON-looking string failed to parse: %s", val)
                                 result[key] = val
                         else:
                             result[key] = val
@@ -308,7 +314,7 @@ class DynamoDbClient:
             for key, val_dict in dynamo_row.items():
                 for val_type, val in val_dict.items():
 
-                    # type_deserializer.deserialize() parses 'N' to `Decimal` type but it cant be parsed to a datetime
+                    # type_deserializer.deserialize() parses 'N' to ``Decimal`` type but it cant be parsed to a datetime
                     # so we cast it to either an integer or a float.
                     if val_type == 'N':
                         result[key] = float(val) if '.' in val else int(val)
@@ -321,14 +327,14 @@ class DynamoDbClient:
                             try:
                                 result[key] = json.loads(val)
                             except ValueError:
-                                logger.warning(f"A JSON-looking string failed to parse: {val}")
+                                logger.warning("A JSON-looking string failed to parse: %s", val)
                                 result[key] = val
                         else:
                             result[key] = val
                     else:
                         result[key] = self.type_deserializer.deserialize(val_dict)
 
-        assert all(True for x in self.config['required_fields'] if result.get(x)), "Some `required_fields` are missing"
+        assert all(True for x in self.config['required_fields'] if result.get(x)), "Some ``required_fields`` are missing"
         return result
 
 
@@ -336,14 +342,17 @@ class DynamoDbClient:
         """
         Convert the row from regular dictionary to the ugly DynamoDB syntax. Takes settings from row_mapper.
 
-        e.g.                {'key1': 'value1', 'key2': 'value2'}
-        will convert to:    {'key1': {'Type1': 'value1'}, 'key2': {'Type2': 'value2'}}
+        e.g.                ``{'key1': 'value1', 'key2': 'value2'}``
+        will convert to:    ``{'key1': {'Type1': 'value1'}, 'key2': {'Type2': 'value2'}}``
 
         :param dict row_dict:   A row we want to convert to dynamo syntax.
         :param str add_prefix:  A string prefix to add to the key in the result dict. Useful for queries like update.
         :param bool strict:     If False, will get the type from the value in the dict (this works for numbers and
                                 strings). If True, won't add them if they're not in the required_fields, and if they
-                                are, will raise an error.
+                                are, will raise an error. Uses ``boto3.types.TypeSerializer`` for type conversion,
+                                but before that automatically guesses that numeric values should become ``N`` type,
+                                boolean types or 'true/false' strings - ``B``, dictionaries - ``M`` recursively.
+
 
         :return:                DynamoDB Task item
         :rtype:                 dict
@@ -379,9 +388,10 @@ class DynamoDbClient:
             if not strict:
                 val = row_dict.get(key)
                 key_with_prefix = f"{add_prefix}{key}"
-                if isinstance(val, bool):
+                if isinstance(val, bool) or (isinstance(val, str) and val.lower() in ['false', 'true']):
                     result[key_with_prefix] = {'BOOL': to_bool(val)}
-                elif isinstance(val, (int, float)) or (isinstance(val, str) and val.isnumeric()):
+                elif isinstance(val, (int, float)) or (isinstance(val, str)
+                                                       and (val.isnumeric() or val.replace('.', '', 1).isnumeric())):
                     result[key_with_prefix] = {'N': str(val)}
                 elif isinstance(val, str):
                     result[key_with_prefix] = {'S': str(val)}
@@ -391,68 +401,71 @@ class DynamoDbClient:
                     result[key_with_prefix] = self.type_serializer.serialize(val)
             else:
                 if key not in self.config.get('required_fields', []):
-                    logger.warning(f"Field {key} is missing from row_mapper, so we can't convert it to DynamoDB "
-                                   f"syntax. This is not a required field, so we continue, but please investigate "
-                                   f"row: {row_dict}")
+                    logger.warning("Field %s is missing from row_mapper, so we can't convert it to DynamoDB "
+                                   "syntax. This is not a required field, so we continue, but please investigate "
+                                   "row: %s", key, row_dict)
                 else:
                     raise ValueError(f"Field {key} is missing from row_mapper, so we can't convert it to DynamoDB "
                                      f"syntax. This is a required field, so we can not continue. Row: {row_dict}")
 
-        logger.debug(f"dict_to_dynamo result: {result}")
+        logger.debug("dict_to_dynamo result: %s", result)
         return result
 
-    def get_by_query(self, keys: Dict, table_name: Optional[str] = None, index_name: Optional[str] = None,
-                     comparisons: Optional[Dict] = None, max_items: Optional[int] = None,
-                     filter_expression: Optional[str] = None, strict: bool = None, return_count: bool = False,
-                     desc: bool = False, fetch_all_fields: bool = None, expr_attrs_names: list = None,
-                     consistent_read: bool = None) -> Union[List[Dict], int]:
-        """
-        Get an item from a table, by some keys. Can specify an index.
-        If an index is not specified, will query the table.
-        IMPORTANT: You must specify the rows you expect to be converted in row mapper in config, otherwise you won't
-        get them in the result.
-        If you want to get items from dynamo by non-key attributes, this method is not for you.
 
-        :param dict keys: Keys and values of the items we get.
-            You must specify the hash key, and can optionally also add the range key.
-            Example, in a table where the hash key is 'hk' and the range key is 'rk':
-            * {'hk': 'cat', 'rk': '123'}
-            * {'hk': 'cat'}
+    def _query_constructor(self, keys: Dict,
+                           table_name: Optional[str] = None,
+                           *,
+                           index_name: Optional[str] = None,
+                           comparisons: Optional[Dict] = None,
+                           max_items: Optional[int] = None,
+                           filter_expression: Optional[str] = None,
+                           strict: bool = None,
+                           return_count: bool = False,
+                           desc: bool = False,
+                           fetch_all_fields: bool = None,
+                           expr_attrs_names: list = None,
+                           consistent_read: bool = None) -> dict:
+        """
+        ..  _query_constructor:
+
+        Constructs a query to retrieve items from a DynamoDB table based on specified parameters.
+        Can specify an index. If an index is not specified, will query the table.
+
+        :param dict keys:   Keys and values to use in query.
+                            You must specify the hash key, and can optionally also add the range key.
 
         Optional
 
         :param str table_name:  Name of the dynamo table. If not specified, will use table_name from the config.
         :param str index_name:  Name of the secondary index in the table. If not specified, will query the table itself.
         :param dict comparisons: Type of comparison for each key. If a key is not mentioned, comparison type will be =.
-            Valid values: `=`, `<`, `<=`, `>`, `>=`, `begins_with`.
+            Valid values: ``=``, ``<``, ``<=``, ``>``, ``>=``, ``begins_with``.
             Comparisons only work for the range key.
-            Example: if keys={'hk': 'cat', 'rk': 100} and comparisons={'rk': '<='} -> will get items where rk <= 100
+            Example: if ``keys={'hk': 'cat', 'rk': 100} and comparisons={'rk': '<='}`` -> will get items where
+            ``rk <= 100``
 
         :param int max_items:   Limit the number of items to fetch.
         :param str filter_expression:  Supports regular comparisons and `between`. Input must be a regular human string
-            e.g. 'key <= 42', 'name = marta', 'foo between 10 and 20', etc.
-        :param bool strict: DEPRECATED.
-        :param bool return_count: If True, will return the number of items in the result instead of the items themselves
-        :param bool desc:    By default (False) the the values will be sorted ascending by the SortKey.
-                             To reverse the order set the argument `desc = True`.
+            e.g. ``'key <= 42', 'name = marta', 'foo between 10 and 20'``, etc.
+        :param bool desc:    By default, (False) the values will be sorted ascending by the SortKey.
+                             To reverse the order set the argument ``desc=True``.
         :param bool fetch_all_fields: If False, will only get the attributes specified in the row mapper.
                                       If True, will get all attributes. Default is False.
         :param list expr_attrs_names: List of attributes names, in case if an attribute name begins with a number or
             contains a space, a special character, or a reserved word, you must use an expression attribute name to
             replace that attribute's name in the expression.
             Example, if the list ['session', 'key'] is received, then a new dict will be assigned to
-            `ExpressionAttributeNames`:
-            {'#session': 'session', '#key': 'key'}
+            ``ExpressionAttributeNames``:
+            ``{'#session': 'session', '#key': 'key'}``
         :param bool consistent_read: If True , then the operation uses strongly consistent reads;
             otherwise, the operation uses eventually consistent reads. Default is False
 
-        :return: List of items from the table, each item in key-value format
-            OR the count if `return_count` is True
+        :return: Query parameters for boto3 Dynamo DB query
         """
 
         if strict is not None:
-            logging.warning(f"get_by_query `strict` variable is deprecated in sosw 0.7.13+. "
-                            f"Please replace it's usage with `fetch_all_fields` (and reverse the boolean value)")
+            logger.warning("get_by_query ``strict`` variable is deprecated in sosw 0.7.13+. "
+                           "Please replace it's usage with ``fetch_all_fields`` (and reverse the boolean value)")
         fetch_all_fields = fetch_all_fields if fetch_all_fields is not None else False if strict is None else not strict
 
         table_name = self._get_validate_table_name(table_name)
@@ -500,6 +513,8 @@ class DynamoDbClient:
         }
 
         if consistent_read is not None:
+            if index_name is not None and consistent_read is True:
+                raise ValueError(f"Strongly Consistent Read is not available on Global Secondary Indexes.")
             logger.debug("Forcing ConsistentRead in query args of get_by_query to: %s", consistent_read)
             query_args['ConsistentRead'] = consistent_read
 
@@ -522,27 +537,39 @@ class DynamoDbClient:
         if max_items:
             query_args['PaginationConfig'] = {'MaxItems': max_items}
             if return_count:
-                raise Exception(f"DynamoDbCLient.get_by_query does not support `max_items` and `return_count` together")
+                raise Exception(f"DynamoDbCLient.get_by_query does not support ``max_items`` and ``return_count`` together")
 
         if desc:
             query_args['ScanIndexForward'] = False
 
-        logger.debug(f"Querying dynamo: {query_args}")
+        logger.debug("Querying dynamo: %s", query_args)
+
+        return query_args
+
+
+    def get_by_query(self, keys: Dict, **kwargs) -> List[Dict]:
+        """
+        Executes a query to the DynamoDB database using the provided keys and additional parameters.
+
+        For signature description see: query_constructor_
+        """
+
+        query_args = self._query_constructor(keys=keys, **kwargs)
 
         paginator = self.dynamo_client.get_paginator('query')
         response_iterator = paginator.paginate(**query_args)
         result = []
 
-        if return_count:
+        if kwargs.get('return_count'):
             return sum([page['Count'] for page in response_iterator])
 
         for page in response_iterator:
-            result += [self.dynamo_to_dict(x, fetch_all_fields=fetch_all_fields) for x in page['Items']]
+            result += [self.dynamo_to_dict(x, fetch_all_fields=kwargs.get('fetch_all_fields')) for x in page['Items']]
             self.stats['dynamo_get_queries'] += 1
-            if max_items and len(result) >= max_items:
+            if kwargs.get('max_items') and len(result) >= kwargs.get('max_items'):
                 break
 
-        return result[:max_items] if max_items else result
+        return result[:kwargs.get('max_items')] if kwargs.get('max_items') else result
 
 
     def _parse_filter_expression(self, expression: str) -> Tuple[str, Dict]:
@@ -550,7 +577,7 @@ class DynamoDbClient:
         Converts FilterExpression to Dynamo syntax. We still do not support some operators. Feel free to implement:
         https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Expressions.OperatorsAndFunctions.html
 
-        Supported: regular comparators, between, attribute_[not_]exists
+        Supported: regular comparators, between, attribute_[not\_]exists
 
         :return:  Returns a tuple of the transformed expression and extracted variables already Dynamo formatted.
         """
@@ -576,7 +603,7 @@ class DynamoDbClient:
             result_expr = f"{key} {operator} :filter_{key}"
             result_values = self.dict_to_dynamo({f"filter_{key}": words[-1]}, add_prefix=':', strict=False)
 
-        # This must be `between` statement.
+        # This must be ``between`` statement.
         elif len(words) == 5:
             assert (words[1].lower(), words[3].lower()) == ('between', 'and'), \
                 f"Unsupported expression for Filtering: {expression}"
@@ -614,8 +641,8 @@ class DynamoDbClient:
         """
 
         if strict is not None:
-            logging.warning(f"get_by_query `strict` variable is deprecated in sosw 0.7.13+. "
-                            f"Please replace it's usage with `fetch_all_fields` (and reverse the boolean value)")
+            logger.warning("get_by_query ``strict`` variable is deprecated in sosw 0.7.13+. "
+                           "Please replace it's usage with ``fetch_all_fields`` (and reverse the boolean value)")
         fetch_all_fields = fetch_all_fields if fetch_all_fields is not None else False if strict is None else not strict
 
         response_iterator = self._build_scan_iterator(attrs, table_name, index_name, consistent_read)
@@ -651,8 +678,8 @@ class DynamoDbClient:
         """
 
         if strict is not None:
-            logging.warning(f"get_by_query `strict` variable is deprecated in sosw 0.7.13+. "
-                            f"Please replace it's usage with `fetch_all_fields` (and reverse the boolean value)")
+            logger.warning("get_by_query ``strict`` variable is deprecated in sosw 0.7.13+. "
+                           "Please replace it's usage with ``fetch_all_fields`` (and reverse the boolean value)")
         fetch_all_fields = fetch_all_fields if fetch_all_fields is not None else False if strict is None else not strict
 
         response_iterator = self._build_scan_iterator(attrs, table_name, index_name, consistent_read)
@@ -695,7 +722,7 @@ class DynamoDbClient:
         if index_name:
             query_args['IndexName'] = index_name
 
-        logger.debug(f"Scanning dynamo: {query_args}")
+        logger.debug("Scanning dynamo: %s", query_args)
 
         paginator = self.dynamo_client.get_paginator('scan')
         response_iterator = paginator.paginate(**query_args)
@@ -712,14 +739,14 @@ class DynamoDbClient:
         :param list keys_list: A list of the keys of the items we want to get. Gets the items that match the given keys.
                                If some key doesn't exist - it just skips it and gets the others.
                                e.g. [{'hash_col': '1, 'range_col': 2}, {'hash_col': 3}]
-                               - will get a row where `hash_col` is 1 and `range_col` is 2, and also all rows where
-                               `hash_col` is 3.
+                               - will get a row where ``hash_col`` is 1 and ``range_col`` is 2, and also all rows where
+                               ``hash_col`` is 3.
 
         Optional
 
         :param str table_name:
         :param int max_retries: If failed to get some items, retry this many times. Waiting between retries is
-                                multiplied by 2 after each retry, so `retries` shouldn't be a big number.
+                                multiplied by 2 after each retry, so ``retries`` shouldn't be a big number.
                                 Default is 1.
         :param int retry_wait_base_time: Wait this much time after first retry. Will wait twice longer in each retry.
         :param bool strict: DEPRECATED.
@@ -732,8 +759,8 @@ class DynamoDbClient:
         """
 
         if strict is not None:
-            logging.warning(f"batch_get_items_one_table `strict` variable is deprecated in sosw 0.7.13+. "
-                            f"Please replace it's usage with `fetch_all_fields` (and reverse the boolean value)")
+            logger.warning("batch_get_items_one_table ``strict`` variable is deprecated in sosw 0.7.13+. "
+                           "Please replace it's usage with ``fetch_all_fields`` (and reverse the boolean value)")
         fetch_all_fields = fetch_all_fields if fetch_all_fields is not None else False if strict is None else not strict
 
         table_name = self._get_validate_table_name(table_name)
@@ -776,11 +803,11 @@ class DynamoDbClient:
                 retry_num = 0
                 wait_time = retry_wait_base_time
                 while unprocessed_keys and retry_num < max_retries:
-                    logger.warning(f"batch_get_item action did NOT finish successfully.")
+                    logger.warning("batch_get_item action did NOT finish successfully.")
                     time.sleep(wait_time)
                     batch_get_item_query['RequestItems'][table_name]['Keys'] = unprocessed_keys
                     latest_result = self.dynamo_client.batch_get_item(**batch_get_item_query)
-                    logger.debug(f"latest_result: {latest_result}")
+                    logger.debug("latest_result: %s", latest_result)
                     all_items += latest_result['Responses'][table_name]
                     retry_num += 1
                     wait_time *= 2
@@ -841,11 +868,11 @@ class DynamoDbClient:
                 "'create' method, or 'overwrite_existing=False' option."
 
         put_query = self.build_put_query(row, table_name, overwrite_existing)
-        logger.debug(f"Put to DB: {put_query}")
+        logger.debug("Put to DB: %s", put_query)
 
         dynamo_response = self.dynamo_client.put_item(**put_query)
 
-        logger.debug(f"Response from dynamo {dynamo_response}")
+        logger.debug("Response from dynamo %s", dynamo_response)
 
         self.stats['dynamo_put_queries'] += 1
 
@@ -853,7 +880,7 @@ class DynamoDbClient:
     def create(self, row: Dict, table_name: str = None):
         """
         Uses the mechanism of the ``put`` method, but first validates that the item with same hash & [range] key[s]
-        does not exist in the table. Otherwise raises: ``ConditionalCheckFailedException``
+        does not exist in the table. Otherwise, raises: ``ConditionalCheckFailedException``
 
         ..  warning:: This method requires the config to have a 'hash_key' parameter with a name of a field.
 
@@ -868,21 +895,21 @@ class DynamoDbClient:
                attributes_to_increment: Optional[Dict] = None, table_name: Optional[str] = None,
                condition_expression: Optional[str] = None, attributes_to_remove: Optional[List[str]] = None):
         """
-        Updates an item in DynamoDB. Will create a new item if doesn't exist.
+        Updates an item in DynamoDB. Will create a new item if it doesn't exist.
         IMPORTANT - If you want to make sure it exists, use ``patch`` method
 
         :param dict keys:
             Keys and values of the row we update.
             Example, in a table where the hash key is 'hk' and the range key is 'rk':
-            {'hk': 'cat', 'rk': '123'}
+            ``{'hk': 'cat', 'rk': '123'}``
         :param dict attributes_to_update:
             Dict of the attributes to be updated.
             Can contain both existing attributes and new attributes.
             Will update existing, and create new attributes.
-            Example: {'col_name': 'some_value'}
+            Example: ``{'col_name': 'some_value'}``
         :param dict attributes_to_increment:
             Attribute names to increment, and the value to increment by. If the attribute doesn't exist, will create it.
-            Example: {'some_counter': '3'}
+            Example: ``{'some_counter': '3'}``
         :param list attributes_to_remove: Will remove these attributes from the record
         :param str condition_expression: Condition Expression that must be fulfilled on the object to update.
         :param str table_name: Name of the table
@@ -942,9 +969,9 @@ class DynamoDbClient:
                 update_item_query['ExpressionAttributeValues'] = update_item_query.get('ExpressionAttributeValues', {})
                 update_item_query['ExpressionAttributeValues'].update(values)
 
-        logger.debug(f"Updating an item, query: {update_item_query}")
+        logger.debug("Updating an item, query: %s", update_item_query)
         response = self.dynamo_client.update_item(**update_item_query)
-        logger.debug(f"Update result: {response}")
+        logger.debug("Update result: %s", response)
         self.stats['dynamo_update_queries'] += 1
 
 
@@ -988,7 +1015,7 @@ class DynamoDbClient:
         Will split transactions to chunks - because transact_write_items accepts up to 10 actions.
         WARNING: If you're expecting a transaction on more than 10 operations - AWS DynamoDB doesn't support it.
 
-        .. code-block:: python
+        ..  code-block:: python
 
             dynamo_db_client = DynamoDbClient(config)
             t1 = dynamo_db_client.make_put_transaction_item(row, table_name='table1')
@@ -1008,12 +1035,12 @@ class DynamoDbClient:
                                                 f"{type(t[action])}"
 
         for t_chunk in chunks(transactions, 10):
-            logger.debug(f"Transactions: \n{pprint.pformat(t_chunk)}")
+            logger.debug("Transactions: %s", t_chunk)
 
             response = self.dynamo_client.transact_write_items(TransactItems=t_chunk)
 
             self.stats['dynamo_transact_write_operations'] += 1
-            logger.debug(f"Response from transact_write_items: {response}")
+            logger.debug("Response from transact_write_items: %s", response)
 
 
     def _get_validate_table_name(self, table_name=None):
@@ -1049,7 +1076,7 @@ class DynamoDbClient:
         """
 
         if table_name is None:
-            logging.debug(self.config)
+            logger.debug(self.config)
             table_name = self.config['table_name']
 
         if table_name not in self._table_descriptions:
@@ -1072,7 +1099,7 @@ class DynamoDbClient:
         """
 
         if table_name is None:
-            logging.debug(self.config)
+            logger.debug(self.config)
             table_name = self.config['table_name']
 
         # No need to sleep for ON DEMAND (PAY_PER_REQUEST) tables.
@@ -1092,7 +1119,7 @@ class DynamoDbClient:
         time_to_sleep = time_between_actions - time_elapsed
 
         if time_to_sleep > 0:
-            logging.debug(f"Sleeping {time_to_sleep} sec")
+            logger.debug("Sleeping %s sec", time_to_sleep)
             time.sleep(time_to_sleep)
 
 
@@ -1110,10 +1137,12 @@ def clean_dynamo_table(table_name='autotest_dynamo_db', keys=('hash_col', 'range
     :param str table_name: name of the table
     :param tuple keys: the keys of the table
     :param str filter_expression:  Supports regular comparisons and `between`. Input must be a regular human string
-        e.g. 'key <= 42', 'name = marta', 'foo between 10 and 20', etc.
+        e.g. ``'key <= 42', 'name = marta', 'foo between 10 and 20'``, etc.
 
-    .. warning:: There are some reserved words that woud not work with
-                 Filter Expression in case they are attribute names. Fix this one day.
+    ..  warning::
+
+        There are some reserved words that would not work with
+        Filter Expression in case they are attribute names. Fix this one day.
 
     """
 
@@ -1149,6 +1178,6 @@ def clean_dynamo_table(table_name='autotest_dynamo_db', keys=('hash_col', 'range
             )
 
             stats['deleted'] += 1
-        logger.debug(f"clean_dynamo_table() of '{table_name}': {stats}")
+        logger.debug("clean_dynamo_table() of %s: %s", table_name, stats)
 
-    logger.info(f"clean_dynamo_table() of '{table_name}': {stats}")
+    logger.info(f"clean_dynamo_table() of %s: %s", table_name, stats)
